

#############################################################
#####         low level utility routines       ##############
#############################################################
  
namevalue <- function(nam, val) {
  paste(nam, "=\"", val, "\"", sep="") 
  }

numbered.id <- function(before, g, after) {
  paste("id=\"", before, g, after, "\"", sep="") 
  }
  
id <- function(id) { 
  paste("id=\"", id, "\"", sep="")
  }
  
  
idref <- function(element, id) {
  paste("<", element, " idref=\"", id, "\" />", sep="") 
  }  

numbered.idref <- function(element, before, g, after) {
  paste("<", element, " idref=\"", before, g, after, "\" />", sep="") 
  }  
  

catln <- function(indent, ...) {
  if (indent == 0) {
    cat(..., "\n", sep="")
    } else {
    tabs <- paste(rep("\t", indent), sep="", collapse="")
    cat(tabs, ..., "\n") 
    }
  }
 
commentln <- function(txt) {
cat("<!-- ", txt, " -->\n"); 
}

  
#############################################################
##########   functions for real data           ##############
#############################################################
 

noftetraploids.fromTable <- function(taxatable) {
noftetraploids <- 0
spp <- unique(taxatable[,"species"])
for (sp in 1:length(spp)) {
  wsp <- which(taxatable[,"species"] == spp[sp])
  ABs <- taxatable[wsp,"genome"]
  if (length(grep("B", ABs)) > 0) {
    noftetraploids <- noftetraploids + 1
    }
  }
noftetraploids
}


make.taxa.and.alignments.fromRealData <- function(taxatable, alignments, alignmentnames) {

commentln("Taxa. Each taxon is a sequence (A or B) from an individual")
commentln("in a species.")
  
ntaxaIds <- nrow(taxatable)
catln(1, "<taxa id=\"taxa\">")
for (sq in 1:ntaxaIds) {
  catln(2, "<taxon",  id(taxatable[sq,"ID"]), "/>")
  }
catln(1, "</taxa>")

nofalignments <- length(alignments)
for (g in 1:nofalignments) {
  commentln(paste("Alignment for gene", g, "from nex file", alignmentnames[g]))
  catln(1, "<alignment", id(paste("alignment", g, sep="")), "dataType=\"nucleotide\">")
  algn <- alignments[[g]]
  seqlen <- length(algn[[1]])
  anames <-  names(algn)
  for (sq in 1:ntaxaIds) {
    catln(2, "<sequence>")
    ID <- taxatable[sq,"ID"]
    catln(3, idref("taxon", ID))
    gotseq <- FALSE
    for (n in 1:length(anames)) {
      if (anames[n] == ID) {
        cat("\t\t\t", toupper(algn[[n]]), "\n", sep="")
         gotseq <- TRUE
        }
      }
    if (!gotseq) {
      cat("\t\t\t", rep("?", seqlen), "\n", sep="")
      }
    catln(2, "</sequence>")
    }
  catln(1, "</alignment>")
  }
}

 

make.xml.realdata <- function(mulTreeVersion, taxatable, alignments, beastNETorMULxmlinfo, beastxmlinfo) {
           
dpath <- beastNETorMULxmlinfo$data.dpath           
nofalignments <- length(beastNETorMULxmlinfo$alignmentnames)

sink(beastxmlinfo$beastXMLfilename)
make.beastxml.start(paste("Input data from", dpath))
make.taxa.and.alignments.fromRealData(taxatable, alignments, beastNETorMULxmlinfo$alignmentnames)
commentln("Patterns")
for (g in 1:nofalignments) {
  make.patterns(g)
  }

nofdiploids <- 2
noftetraploids <- noftetraploids.fromTable(taxatable)
noftetratrees <- 1
    
make.beastxml.after.patterns(nofdiploids, noftetraploids, noftetratrees, nofalignments, 
                  taxastruct=NULL, taxatable,
                  beastxmlinfo$sampledgtrees.fpathbase,
                  beastxmlinfo$sampledmultrees.fpath,
                  beastxmlinfo$sampledparams.fpath,
                  beastxmlinfo$DBUGTUNE.fpath,
                  mulTreeVersion)
sink(NULL)
}



make.beastxml.MULandNET.forRealData <- function(beastNETorMULxmlinfo) {

dpath <- beastNETorMULxmlinfo$data.dpath
beastNETxmlinfo <- list(                                         
    beastXMLfilename=       paste(dpath, "NET", beastNETorMULxmlinfo$beastXMLfilename, sep=""),
    sampledgtrees.fpathbase=paste(dpath, "NET", beastNETorMULxmlinfo$sampledgtrees.fnamebase, sep=""),
    sampledmultrees.fpath=  paste(dpath, "NET", beastNETorMULxmlinfo$sampledmultrees.fname, sep=""),
    sampledparams.fpath=    paste(dpath, "NET", beastNETorMULxmlinfo$sampledparams.fname, sep=""),
    DBUGTUNE.fpath=         paste(dpath, "NET", beastNETorMULxmlinfo$DBUGTUNE.fname , sep=""))

beastMULxmlinfo <- list(                                         
    beastXMLfilename=       paste(dpath, "MUL", beastNETorMULxmlinfo$beastXMLfilename, sep=""),
    sampledgtrees.fpathbase=paste(dpath, "MUL", beastNETorMULxmlinfo$sampledgtrees.fnamebase, sep=""),
    sampledmultrees.fpath=  paste(dpath, "MUL", beastNETorMULxmlinfo$sampledmultrees.fname, sep=""),
    sampledparams.fpath=    paste(dpath, "MUL", beastNETorMULxmlinfo$sampledparams.fname, sep=""),
    DBUGTUNE.fpath=         paste(dpath, "MUL", beastNETorMULxmlinfo$DBUGTUNE.fname , sep=""))

nofalignments <- length(beastNETorMULxmlinfo$alignmentnames)
alignments <- NULL
for (a in 1:nofalignments) {
  aln <- read.nexus.data(paste(dpath, alignmentnames[a], ".nex", sep=""))
  alignments <- c(alignments, list(aln))
  }
  
taxatable <- read.table(paste(dpath, beastNETorMULxmlinfo$fpath.taxatable, sep=""), header=TRUE, colClasses="character")
stopifnot(length(unique(taxatable[,"ID"])) == length(taxatable[,"ID"]))
ABs <- sort(unique(taxatable[,"genome"]))
stopifnot(length(ABs) == 2)
stopifnot(ABs[1] == "A"  &&  ABs[2] == "B")

make.xml.realdata(FALSE, taxatable, alignments, beastNETorMULxmlinfo, beastNETxmlinfo)
make.xml.realdata(TRUE,  taxatable, alignments, beastNETorMULxmlinfo, beastMULxmlinfo)
}




 
  
  
#############################################################
##########           models and priors         ##############
#############################################################
  

make.patterns <- function(g) {
  catln(1, "<patterns", numbered.id("", g, ".patterns"), namevalue("from", 1), ">")
  catln(2, numbered.idref("alignment", "alignment", g, ""))
  catln(1, "</patterns>")
  }
  
make.constantSize <- function() { 
 	catln(1, "<constantSize",  id("constant"), namevalue("units", "substitutions"), ">")
  catln(2, "<populationSize>")
  catln(3, "<parameter",  id("constant.popSize"),  namevalue("value", 0.016),
                 namevalue("lower", 0.0), namevalue("upper", "Infinity"), "/>")
  catln(2, "</populationSize>")
  catln(1, "</constantSize>")
 }
  
make.coalescentTree <- function(g, roothgt) {
  catln(1, "<coalescentTree ", numbered.id("", g, ".startingTree"),
          namevalue("rootHeight", roothgt), ">")
  catln(2, idref("taxa", "taxa"))
  catln(2, idref("constantSize", "constant"))
  catln(1, "</coalescentTree>")
  }


make.treeModel <- function(g) {
  catln(1, "<treeModel", numbered.id("", g, ".treeModel"), ">")
  catln(2, numbered.idref("coalescentTree", "", g, ".startingTree"))
  
  catln(2, "<rootHeight>")
  catln(3, "<parameter", numbered.id("", g, ".treeModel.rootHeight"), "/>")
  catln(2, "</rootHeight>")
  
  catln(2, "<nodeHeights internalNodes=\"true\" >")
  catln(3, "<parameter", numbered.id("", g, ".treeModel.internalNodeHeights"), "/>")
  catln(2, "</nodeHeights>")

  catln(2, "<nodeHeights internalNodes=\"true\" rootNode=\"true\" >")
  catln(3, "<parameter", numbered.id("", g, ".treeModel.allInternalNodeHeights"), "/>")
  catln(2, "</nodeHeights>")
  
  catln(1, "</treeModel>")
  }



make.strictClockBranchRate <- function(g) {
  catln(1, "<strictClockBranchRates", numbered.id("", g, ".branchRates"), ">")
  catln(2, "<rate>")
  catln(3, "<parameter", numbered.id("", g, ".clock.rate"), 
           namevalue("value", 1.0), namevalue("lower", 0.0),
           namevalue("upper", "Infinity"), "/>") 
  catln(2, "</rate>")
  catln(1, "</strictClockBranchRates>")
  }


  
make.HKYModel <- function(g) {
  catln(1, "<HKYModel", numbered.id("", g, ".hky"), ">")
  catln(2, "<frequencies>")
  catln(3, "<frequencyModel", namevalue("dataType", "nucleotide"), ">")
  catln(4, "<frequencies>")
  catln(5, "<parameter", numbered.id("", g, ".frequencies"), 
                 namevalue("value", "0.25 0.25 0.25 0.25"), "/>")
  catln(4, "</frequencies>")
  catln(3, "</frequencyModel>")
  catln(2, "</frequencies>")
  catln(2, "<kappa>")
  catln(3, "<parameter", numbered.id("", g, ".kappa"), namevalue("value", 2.0),
              namevalue("lower", 0), namevalue("upper", "Infinity"), "/>")
  catln(2, "</kappa>")
  catln(1, "</HKYModel>")
  }


make.siteModel <- function(g) {
  catln(1, "<siteModel", numbered.id("", g, ".siteModel"), ">")
  catln(2, "<substitutionModel>")
  catln(3, numbered.idref("HKYModel", "", g, ".hky"))
  catln(2, "</substitutionModel>")
  catln(1, "</siteModel>")
  }


make.treeLikelihood <- function(g) {
  catln(1, "<treeLikelihood", numbered.id("", g, ".treeLikelihood"), 
                namevalue("useAmbiguities", "false"), ">")
  catln(2, numbered.idref("patterns", "", g, ".patterns"))
  catln(2, numbered.idref("treeModel", "", g, ".treeModel"))
  catln(2, numbered.idref("siteModel", "", g, ".siteModel"))
  catln(2, numbered.idref("strictClockBranchRates", "", g, ".branchRates"))
  catln(1, "</treeLikelihood>")
  }




make.species.list.fromStruct <- function(taxastruct) {
for (sp in 1:length(taxastruct)) {
  spname <- taxastruct[[sp]]$sp
  ploidylevel <- 2 * length(taxastruct[[sp]]$sqs)
  catln(2, "<apsp", id(spname), namevalue("ploidylevel", ploidylevel), ">")
  for (iv in 1:length(taxastruct[[sp]]$indivs)) {
    ivname <- paste(taxastruct[[sp]]$indivs[iv], spname, sep="")
    catln(3, "<individual", id(ivname), ">")
    for (sq in 1:length(taxastruct[[sp]]$sqs)) {
      txname <- paste(ivname, taxastruct[[sp]]$sqs[sq], sep="")
      catln(4, idref("taxon", txname)) 
      }
    catln(3, "</individual>")
    }
  catln(2, "</apsp>")
  }
}
  



make.species.list.fromTable <- function(taxatable) {
spp <- unique(taxatable[,"species"])
for (sp in 1:length(spp)) {
  wsp <- which(taxatable[,"species"] == spp[sp])
  ploidylevel <- 2 * length(unique(taxatable[wsp,"genome"]))
  stopifnot(ploidylevel==2  ||  ploidylevel==4)
  catln(2, "<apsp", id(spp[sp]), namevalue("ploidylevel", ploidylevel), ">")
  indivs <- unique(taxatable[wsp,"individual"])
  for (iv in 1:length(indivs)) {
    wiv <- which((taxatable[,"species"] == spp[sp])  &  (taxatable[,"individual"] == indivs[iv]))
    stopifnot(2 * length(wiv) == ploidylevel)
    catln(3, "<individual", id(paste(spp[sp], indivs[iv], sep="")), ">")
    for (sq in 1:length(wiv)) {
      catln(4, idref("taxon", taxatable[wiv[sq],"ID"])) 
      }
    catln(3, "</individual>")
    }
  catln(2, "</apsp>")
  }
}

  
make.species.list <- function(taxastruct, taxatable) {  
if (is.null(taxastruct)) {
  make.species.list.fromTable(taxatable)        
  } else {
  make.species.list.fromStruct(taxastruct)        
  }
}

  
make.genetree.list <- function(nofgenes) {
catln(2, "<geneTrees", id("geneTrees"), ">")
for (g in 1:nofgenes) {
  catln(3, numbered.idref("treeModel", "", g, ".treeModel"))
  }
catln(2, "</geneTrees>")
}


make.alloppSpecies <- function(nofgenes, taxastruct, taxatable, mingenenodeheight) {
  catln(1, "<alloppSpecies", id("alloppSpecies"), 
               namevalue("minGeneNodeHeight", mingenenodeheight), ">")
  make.species.list(taxastruct, taxatable)
  make.genetree.list(nofgenes)
  catln(1, "</alloppSpecies>")
  } 
  
  
make.mulSpecies <- function(nofgenes, taxastruct, taxatable) {
  catln(1, "<mulSpecies", id("mulSpecies"), ">")
  make.species.list(taxastruct, taxatable)        
  make.genetree.list(nofgenes)
  catln(1, "</mulSpecies>")
  }   
  
  
  
 # grjtodo allow for > 1 hybridizations 
make.alloppSpeciesNetwork <- function(initpopval) { 
  catln(1, "<alloppSpeciesNetwork", id("apspnetwork"), "oneHybridization=\"true\" >")
  catln(2, idref("alloppSpecies", "alloppSpecies")) 
  catln(2, "<sppSplitPopulations", namevalue("value", initpopval), ">")
  catln(3, "<parameter", id("apspnetwork.splitPopSize"), "/>")
  catln(2, "</sppSplitPopulations>") 
  catln(1, "</alloppSpeciesNetwork>")
  }


make.mulSpeciesTree <- function(initpopval) { 
  catln(1, "<mulSpeciesTree", id("mulSpeciesTree"), "constantRoot=\"true\" >")
  catln(2, idref("mulSpecies", "mulSpecies")) 
  catln(2, "<sppSplitPopulations", namevalue("value", initpopval), ">")
  catln(3, "<parameter", id("mulSpeciesTree.splitPopSize"), "/>")
  catln(2, "</sppSplitPopulations>") 
  catln(1, "</mulSpeciesTree>")
  }
  

make.alloppNetworkPriorModel <- function() { 
  catln(1, "<alloppNetworkPriorModel", id("one.rate"), 
                   namevalue("units", "substitutions"), ">")
  catln(2, "<eventRate>")
  catln(3, "<parameter", id("apspnetwork.prior.eventRate"), 
             namevalue("value", 100), namevalue("lower", 0.0), namevalue("upper", "Infinity"),"/>")
  catln(2, "</eventRate>")
  catln(1, "</alloppNetworkPriorModel>")
  }
  

make.yuleMultreePriorModel <- function() { 
  catln(1, "<yuleModel", id("yule"), 
                   namevalue("units", "substitutions"), ">")
  catln(2, "<birthRate>")
  catln(3, "<parameter", id("mulSpecies.yule.birthRate"), 
             namevalue("value", 100), namevalue("lower", 0.0), namevalue("upper", "Infinity"),"/>")
  catln(2, "</birthRate>")
  catln(1, "</yuleModel>")
  }  


make.apspNetworkPrior <- function() { 
  catln(1, "<apspNetworkPrior", id("apspnetwork.prior"), ">")
  catln(2, "<model>")
  catln(3, idref("alloppNetworkPriorModel", "one.rate"))
  catln(2, "</model>")
  catln(2, "<apspNetwork>")
  catln(3, idref("alloppSpeciesNetwork", "apspnetwork"))
  catln(2, "</apspNetwork>")
  catln(1, "</apspNetworkPrior>")
  }
  
  
make.mulSpeciesTreePrior <- function() { 
  catln(1, "<mulSpeciesTreePrior", id("mulSpeciesTree.prior"), ">")
  catln(2, "<model>")
  catln(3, idref("yuleModel", "yule"))
  catln(2, "</model>")
  catln(2, "<mulTree>")
  catln(3, idref("mulSpeciesTree", "mulSpeciesTree"))
  catln(2, "</mulTree>")
  catln(1, "</mulSpeciesTreePrior>")
  }  
  
  
  
make.apspCoalescent <- function() { 
  catln(1, "<apspCoalescent", id("apsp.coalescent"), ">")
  catln(2, idref("alloppSpecies", "alloppSpecies"))
  catln(2, idref("alloppSpeciesNetwork", "apspnetwork"))
  catln(1, "</apspCoalescent>")    
  }



make.mulMSCoalescent <- function() { 
  catln(1, "<mulMSCoalescent", id("mulMSCoalescent"), ">")
  catln(2, idref("mulSpecies", "mulSpecies"))
  catln(2, idref("mulSpeciesTree", "mulSpeciesTree"))
  catln(1, "</mulMSCoalescent>")    
  }


make.gammaDistributionModel <- function(shape, initpopval, first) {
  catln(3, "<gammaDistributionModel>")
  catln(4, "<shape>")
  catln(5, shape)
  catln(4, "</shape>")
  catln(4, "<scale>")
  if (first) {
    catln(5, "<parameter", id("species.popMean"), namevalue("value", initpopval), 
               namevalue("lower", 0.0), namevalue("upper", "Infinity"),"/>")
    } else {
    catln(5, idref("parameter", "species.popMean"))
    }
  catln(4, "</scale>")
  catln(3, "</gammaDistributionModel>")
  }



 # this for is two diploids case
make.mixedDistributionLikelihood <- function(nofdiploids, noftetraploids, 
                                         noftetratrees, initpopval, popsize.param.ref, flags) { 
catln(1, "<mixedDistributionLikelihood", id("species.popSize"), ">")
catln(2, "<distribution0>")
make.gammaDistributionModel(2, initpopval, TRUE)
catln(2, "</distribution0>")
catln(2, "<distribution1>")
make.gammaDistributionModel(4, initpopval, FALSE)
catln(2, "</distribution1>")
catln(2, "<data>")
catln(3, idref("parameter", popsize.param.ref))
catln(2, "</data>")
catln(2, "<indicators>")
catln(3, "<parameter",  namevalue("value", flags), "/>")
catln(2, "</indicators>")  
catln(1, "</mixedDistributionLikelihood>")
}
   
   
make.network.mixedDistributionLikelihood <- function(nofdiploids, noftetraploids, 
                                         noftetratrees, initpopval) { 
flags <- ""
for (i in 1:(nofdiploids + noftetraploids)) {
  flags <- paste(flags, "1")
  }
for (i in 1:(2*nofdiploids + 2*noftetraploids + noftetratrees - 2)) {
  flags <- paste(flags, "0")
 }
make.mixedDistributionLikelihood(nofdiploids, noftetraploids, 
                                         noftetratrees, initpopval, "apspnetwork.splitPopSize", flags)
}
    
    
make.multree.mixedDistributionLikelihood <- function(nofdiploids, noftetraploids, 
                                         noftetratrees, initpopval) { 
flags <- ""
for (i in 1:(nofdiploids + 2*noftetraploids)) {
  flags <- paste(flags, "1")
  }
for (i in 1:(2*nofdiploids + 4*noftetraploids - 2)) {
  flags <- paste(flags, "0")
 }
make.mixedDistributionLikelihood(nofdiploids, noftetraploids, 
                                         noftetratrees, initpopval, "mulSpeciesTree.splitPopSize", flags)
}    
                                         

#############################################################
##########            MCMC operators           ##############
#############################################################
 


make.general.numbered.scaleOperator <- function(g, sfactor, weight, before, after) {
  catln(2, "<scaleOperator", namevalue("scaleFactor", sfactor),
         namevalue("weight", weight), ">")
  catln(3, numbered.idref("parameter", before, g, after))
  catln(2, "</scaleOperator>")
  }		
		
make.general.scaleOperator <- function(sfactor, weight, param) {
  catln(2, "<scaleOperator", namevalue("scaleFactor", sfactor),
         namevalue("weight", weight), ">")
  catln(3, idref("parameter", param))
  catln(2, "</scaleOperator>")
  }			
		
		
make.scaleOperator.kappa <- function(g, weight) {
  make.general.numbered.scaleOperator(g, 0.75, weight, "", ".kappa")
  }
  

make.scaleOperator.clockrate <- function(g, weight) {
  make.general.numbered.scaleOperator(g, 0.75, weight, "", ".clock.rate")
  }  
	
make.deltaExchange.frequencies <- function(g, weight) {
  catln(2, "<deltaExchange", namevalue("delta", 0.01),
         namevalue("weight", weight), ">")
  catln(3, numbered.idref("parameter", "", g, ".frequencies"))
  catln(2, "</deltaExchange>")  
  }
  

make.subtreeSlide.treeModel <- function(g, weight) {
  catln(2, "<subtreeSlide", namevalue("size", .002), 
       namevalue("gaussian", "true"), namevalue("weight", weight), ">")
  catln(3, numbered.idref("treeModel", "", g, ".treeModel"))
  catln(2, "</subtreeSlide>") 
  }
  

  
make.narrowExchange.treeModel <- function(g, weight) {
  catln(2, "<narrowExchange", namevalue("weight", weight), ">")
  catln(3, numbered.idref("treeModel", "", g, ".treeModel"))
  catln(2, "</narrowExchange>") 
  }
  
   
make.wideExchange.treeModel <- function(g, weight) {
  catln(2, "<wideExchange", namevalue("weight", weight), ">")
  catln(3, numbered.idref("treeModel", "", g, ".treeModel"))
  catln(2, "</wideExchange>") 
  }
  
   
make.wilsonBalding.treeModel <- function(g, weight) {
  catln(2, "<wilsonBalding", namevalue("weight", weight), ">")
  catln(3, numbered.idref("treeModel", "", g, ".treeModel"))
  catln(2, "</wilsonBalding>") 
  
  }
  
  
make.scaleOperator.treeModel.rootHeight <- function(g, weight) {
  make.general.numbered.scaleOperator(g, 0.75, weight, "", ".treeModel.rootHeight")
  }
  

 
make.uniformOperator.treeModel.internalNodeHeights <- function(g, weight) {
  catln(2, "<uniformOperator", namevalue("weight", weight), ">")
  catln(3, numbered.idref("parameter", "", g, ".treeModel.internalNodeHeights"))
  catln(2, "</uniformOperator>")
  }
  
   

   

   
   
make.upDownOperator.clockrate.allInternalNodeHeights <- function(g, weight) {
  catln(2, "<upDownOperator", namevalue("scaleFactor", 0.75), 
           namevalue("weight", weight), ">")
  catln(3, "<up>")
  if (g>1) {
    catln(4, numbered.idref("parameter", "", g, ".clock.rate")) 
    }  
  catln(3, "</up>")
  catln(3, "<down>")
  catln(4, numbered.idref("parameter", "", g, ".treeModel.allInternalNodeHeights"))
  catln(3, "</down>")
  catln(2, "</upDownOperator>")
  } 
  
  
      
make.network.upDownOperator.clocks.pops.heights <- function(nofgenes, weight) {
  catln(2, "<upDownOperator", namevalue("scaleFactor", 0.75), 
           namevalue("weight", weight), ">")
           
  catln(3, "<up>")
  for (g in 1:nofgenes) {
    if (g>1) {
      catln(4, numbered.idref("parameter", "", g, ".clock.rate")) 
    }
  }
  catln(4, idref("parameter", "apspnetwork.prior.eventRate")) 
  catln(3, "</up>")
  
  catln(3, "<down>")
  catln(4, idref("alloppSpeciesNetwork", "apspnetwork"))
  catln(4, idref("parameter" , "species.popMean"))
  catln(4, idref("parameter" , "apspnetwork.splitPopSize"))
  for (g in 1:nofgenes) {
    catln(4, numbered.idref("parameter", "", g, ".treeModel.allInternalNodeHeights"))
  }
  catln(3, "</down>")
  catln(2, "</upDownOperator>")
  }
  
  

make.mulTree.upDownOperator.clocks.pops.heights <- function(nofgenes, weight) {
  catln(2, "<upDownOperator", namevalue("scaleFactor", 0.75), 
           namevalue("weight", weight), ">")
           
  catln(3, "<up>")
  for (g in 1:nofgenes) {
    if (g>1) {
      catln(4, numbered.idref("parameter", "", g, ".clock.rate")) 
    }
  }
  catln(4, idref("parameter", "mulSpecies.yule.birthRate")) 
  catln(3, "</up>")
  
  catln(3, "<down>")
  catln(4, idref("mulSpeciesTree", "mulSpeciesTree"))
  catln(4, idref("parameter" , "species.popMean"))
  catln(4, idref("parameter" , "mulSpeciesTree.splitPopSize"))
  for (g in 1:nofgenes) {
    catln(4, numbered.idref("parameter", "", g, ".treeModel.allInternalNodeHeights"))
  }
  catln(3, "</down>")
  catln(2, "</upDownOperator>")
  }




make.scaleOperator.species.popMean <- function(weight) {
  make.general.scaleOperator(0.9, weight, "species.popMean")
  }
  
  
   
make.scaleOperator.apspnetwork.splitPopSize <- function(weight) {
  make.general.scaleOperator(0.5, weight, "apspnetwork.splitPopSize")
  }
  
make.scaleOperator.mulSpeciesTree.splitPopSize <- function(weight) {
  make.general.scaleOperator(0.5, weight, "mulSpeciesTree.splitPopSize")
  }
   
   
make.scaleOperator.apspnetwork.prior.eventRate <- function(weight) {
  make.general.scaleOperator(0.5, weight, "apspnetwork.prior.eventRate")
  }
  

make.scaleOperator.mulSpecies.prior.birthRate <- function(weight) {
  make.general.scaleOperator(0.5, weight, "mulSpecies.yule.birthRate")
  }

    
make.networkNodeReHeightOperator <- function(weight) {
  catln(2, "<networkNodeReHeight",  namevalue("weight", weight), ">")
  catln(3, idref("alloppSpecies", "alloppSpecies"))
  catln(3, idref("alloppSpeciesNetwork", "apspnetwork"))
  catln(2, "</networkNodeReHeight>")   
  }
  
make.mulTreeNodeReHeightOperator <- function(weight) {
  catln(2, "<mulTreeNodeReHeight",  namevalue("weight", weight), ">")
  catln(3, idref("mulSpecies", "mulSpecies"))
  catln(3, idref("mulSpeciesTree", "mulSpeciesTree"))
  catln(2, "</mulTreeNodeReHeight>")   
  }
  
  
make.sequenceReassignmentOperator <- function(weight) {
  catln(2, "<sequenceReassignment",  namevalue("weight", weight), ">")
  catln(3, idref("alloppSpecies", "alloppSpecies"))
  catln(3, idref("alloppSpeciesNetwork", "apspnetwork"))
  catln(2, "</sequenceReassignment>")   
  }

make.mulTreesequenceReassignmentOperator <- function(weight) {
  catln(2, "<mulTreesequenceReassignment",  namevalue("weight", weight), ">")
  catln(3, idref("mulSpecies", "mulSpecies"))
  catln(3, idref("mulSpeciesTree", "mulSpeciesTree"))
  catln(2, "</mulTreesequenceReassignment>")   
  }
  
    
make.moveLegsOperator <- function(weight) {
  catln(2, "<moveLegs",  namevalue("weight", weight), ">")
  catln(3, idref("alloppSpecies", "alloppSpecies"))
  catln(3, idref("alloppSpeciesNetwork", "apspnetwork"))
  catln(2, "</moveLegs>")   
  }
      

